<?php

namespace Drupal\purest_content\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\State\StateInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityFieldManager;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Url;

/**
 * Class NormalizerSettingsForm.
 */
class NormalizerSettingsForm extends ConfigFormBase {

  /**
   * Drupal\Core\State\StateInterface definition.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * Drupal\Core\Config\ConfigFactoryInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The product storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $entityStorage;

  /**
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityTypeBundle;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * Constructs a new ConfigForm object.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StateInterface $state, EntityTypeManagerInterface $type_manager, EntityTypeBundleInfo $type_bundle, EntityFieldManager $entity_field_manager) {
    parent::__construct($config_factory, $state);
    $this->configFactory = $config_factory;
    $this->state = $state;
    $this->entityTypeManager = $type_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeBundle = $type_bundle;

    $this->entity_types = [
      'node' => $this->entityTypeBundle->getBundleInfo('node'),
      'taxonomy_term' => $this->entityTypeBundle->getBundleInfo('taxonomy_term')
    ];
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('state'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity_field.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'purest_content.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'purest_content_settings_form';
  }

  public function getTitle() {
    $bundle = \Drupal::routeMatch()->getParameter('bundle');
    return ucfirst(str_replace('_', ' ', $bundle)) . ' Resource Settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $type = NULL, $bundle = NULL) {
    $config = $this->configFactory->get('purest_content.settings');
    $this->type = $type;
    $this->bundle = $bundle;

    if (!$type || !$bundle) {
      return new RedirectResponse(\Drupal::url('purest_content.config'));
    }

    $bundle_fields = $this->entityFieldManager->getFieldDefinitions($type, $bundle);
  
    $form[$bundle] = [
      '#type' => 'field_group',
      '#title' => ucfirst($bundle) . ' [' . $type . ']',
    ];

    $form[$bundle][$bundle . '_normalize_heading'] = [
      '#type' => 'html_tag',
      '#tag' => 'h3',
      '#value' => $this
        ->t('Purest Normalizer'),
    ];

    $form[$bundle][$bundle . '_normalize'] = [
      '#title' => $this->t('Use Purest normalizer'),
      '#description' => $this
        ->t('Purest inludes a general node and taxonomy term normalizer. It is opinionated but vastly simplifies the complex JSON that the Rest modules provides out of the box.'),
      '#type' => 'checkbox',
      '#default_value' => NULL === $config->get($bundle . '_normalize') ?
                                    1 : $config->get($bundle . '_normalize'),
      '#suffix' => '<br>'
    ];

    $form[$bundle][$bundle . '_remove_empty'] = [
      '#title' => $this->t('Remove Empty Fields'),
      '#description' => $this
        ->t('Remove fields from rest responses when value is null or empty.'),
      '#type' => 'checkbox',
      '#default_value' => NULL === $config->get($bundle . '_remove_empty') ?
                                    1 : $config->get($bundle . '_remove_empty'),
      '#suffix' => '<br>'
    ];

    $form[$bundle][$bundle . '_fields_heading'] = [
      '#type' => 'html_tag',
      '#tag' => 'h3',
      '#value' => $this
        ->t('Field Settings')
    ];

    $form[$bundle][$bundle . '_fields'] = [
      '#title' => $this->t('Entity Fields'),
      '#type' => 'table',
      '#sticky' => TRUE,
      '#header' => [
        $this->t('Name'),
        $this->t('Label'),
        $this->t('Type'),
        $this->t('Custom Label'),
        $this->t('Exclude'),
        $this->t('Operations'),
      ],
    ];

    $values = $config->get($bundle . '_fields');

    foreach ($bundle_fields as $field_name => $field_definition) {
      $form[$bundle][$bundle . '_fields'][$field_name]['name'] = [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => $field_definition->getLabel()
      ];

      $form[$bundle][$bundle . '_fields'][$field_name]['label'] = [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => $field_name
      ];

      $form[$bundle][$bundle . '_fields'][$field_name]['type'] = [
        '#type' => 'textfield',
        '#disabled' => TRUE,
        '#size' => 30,
        '#default_value' => $field_definition->getType()
      ];

      $form[$bundle][$bundle . '_fields'][$field_name]['custom_label'] = [
        '#type' => 'textfield',
        '#size' => 30,
        '#default_value' => NULL !== $values[$field_name]['custom_label'] ?
                                $values[$field_name]['custom_label'] : ''
      ];

      $form[$bundle][$bundle . '_fields'][$field_name]['exclude'] = [
        '#type' => 'checkbox',
        '#default_value' => NULL !== $values[$field_name]['exclude'] ? 
                                $values[$field_name]['exclude'] : 0
      ];

      switch ($field_definition->getType()) {
        case 'entity_reference':
        case 'file':
        case 'image':
          $form[$bundle][$bundle . '_fields'][$field_name]['actions'] = [
            '#type' => 'dropbutton',
            '#links' => [
              'edit' => [
                'title' => $this
                  ->t('Edit'),
                'url' => Url::fromRoute('purest_content.field', [
                  'type' => $type,
                  'bundle' => $bundle,
                  'field' => $field_name
                ]),
              ],
            ],
          ];
          break;
        default :
          $form[$bundle][$bundle . '_fields'][$field_name]['actions'] = [
            '#type' => 'dropbutton',
            '#links' => [],
          ];
      }
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $config = $this->configFactory->getEditable('purest_content.settings');
    $config->set($this->bundle . '_normalize', $form_state->getValue($this->bundle . '_normalize'));
    $config->set($this->bundle . '_remove_empty', $form_state->getValue($this->bundle . '_remove_empty'));
    $config->set($this->bundle . '_fields', $form_state->getValue($this->bundle . '_fields'));

    $config->save();
  }

}
