<?php

namespace Drupal\purest_content\Normalizer;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\serialization\Normalizer\ContentEntityNormalizer;
use Drupal\Core\Access\AccessibleInterface;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\node\NodeInterface;
use Drupal\taxonomy\TermInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\Core\Entity\EntityTypeBundleInfo;
use Drupal\Core\Entity\EntityFieldManager;

/**
 * Converts the Drupal entity object structures to a normalized array.
 */
class ContentNormalizer extends ContentEntityNormalizer {

  /**
   * Config factory
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityTypeBundle;

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManager
   */
  protected $entityFieldManager;

  /**
   * The interface or class that this Normalizer supports.
   *
   * @var string
   */
  protected $supportedInterfaceOrClass = 'Drupal\Core\Entity\ContentEntityInterface';

  /**
   * Constructs an EntityNormalizer object.
   *
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager.
   */
  public function __construct(EntityManagerInterface $entity_manager, ConfigFactoryInterface $config_factory, EntityTypeBundleInfo $entity_type_bundle, EntityFieldManager $entity_field_manager) {
    parent::__construct($entity_manager, $config_factory);
    $this->configFactory = $config_factory;
    $this->config = $this->configFactory->get('purest_content.settings');
    $this->entityManager = $entity_manager;
    $this->entityFieldManager = $entity_field_manager;
    $this->entityTypeBundle = $entity_type_bundle;

    $this->entity_types = [
      'node' => $this->entityTypeBundle->getBundleInfo('node'),
      'taxonomy_term' => $this->entityTypeBundle->getBundleInfo('taxonomy_term')
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function supportsNormalization($data, $format = NULL) {
    // If we aren't dealing with an object or the format is not
    // supported return now.
    if (!is_object($data) || !$this->checkFormat($format)) {
      return FALSE;
    }

    // This custom normalizer should be supported for all entities of type
    // node and taxonomy term
    if ($data instanceof NodeInterface || $data instanceof TermInterface) {
      if ($data instanceof NodeInterface) {
        $bundle = $data->bundle();
        $normalize = $this->config->get($bundle . '_normalize');
        
        if (NULL !== $normalize && !$normalize) {
          return FALSE;
        }
      }

      if ($data instanceof TermInterface) {
        $bundle = $data->getVocabularyId();
        $normalize = $this->config->get($bundle . '_normalize');
        
        if (NULL !== $normalize && !$normalize) {
          return FALSE;
        }
      }
      
      return TRUE;
    }

    // Otherwise, this normalizer does not support the $data object.
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function normalize($entity, $format = NULL, array $context = []) {
    $fields = parent::normalize($entity, $format, $context);
    $type = ($entity instanceof NodeInterface) ? $entity->bundle() : $entity->getVocabularyId();
    $rest_settings = $this->config->get($type . '_fields');
    $hide_empty = NULL === $this->config->get($type . '_hide_empty') ? 
                        1 : intval($this->config->get($type . '_hide_empty'));
    $output = [];

    foreach ($fields as $key => $field) {
      if (isset($rest_settings[$key]) && intval($rest_settings[$key]['exclude'])) {
        continue;
      }

      if (!$field && $hide_empty) {
        continue;
      }

      $label = $key;

      if (isset($rest_settings[$key]['custom_label']) && 
              !empty($rest_settings[$key]['custom_label'])) {
        $label = $rest_settings[$key]['custom_label'];
      }

      switch ($key) {
        case 'type':
          $output[$label] = $fields[$key]['type'];
          break;
        case 'vid':
          $output[$label] = $fields[$key]['target_id'];
          break;
        default:
          $output[$label] = $fields[$key];
      }
    }

    return $output;
  }
}
