<?php

namespace Drupal\purest_menus\Plugin\rest\resource;

use Drupal\Core\Access\AccessibleInterface;
use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\CacheableResponseInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Menu\MenuLinkInterface;
use Drupal\Core\Menu\MenuLinkTreeInterface;
use Drupal\Core\Menu\MenuTreeParameters;
use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Drupal\system\MenuInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Menu\MenuActiveTrailInterface;
use Drupal\Core\Path\AliasStorageInterface;
use Drupal\Core\Language\LanguageManager;
use Drupal\Core\Path\AliasManager;
use Drupal\Core\Url;

/**
 * Provides a resource to retrieve menu trees.
 *
 * @RestResource(
 *   id = "purest_menu_resource",
 *   label = @Translation("Purest Menu Resource"),
 *   serialization_class = "Drupal\system\Entity\Menu",
 *   uri_paths = {
 *     "canonical" = "/purest/menu/{menu}"
 *   }
 * )
 */
class MenuResource extends ResourceBase {

  /**
   * Menu Tree.
   *
   * @var \Drupal\Core\Menu\MenuLinkTreeInterface
   */
  protected $menuTree;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   *
   * 
   */
  protected $alias;

  /**
   * 
   */
  protected $maxDepth;

  /**
   *
   * 
   */
  protected $activeTrail;

  /**
   *
   * 
   */
  protected $aliasManager;

  /**
   *
   * @var \Drupal\Core\Language\LanguageManager
   */
  protected $languageManager;

  /**
   *
   * @var int
   */
  protected $language;

  /**
   *
   * @var \Drupal\Core\Path\AliasStorageInterface
   */
  protected $aliasStorageInterface;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, MenuLinkTreeInterface $menu_tree, MenuActiveTrailInterface $active_trail, EntityTypeManagerInterface $entity_type_manager, Request $current_request, AliasManager $alias_manager, LanguageManager $language_manager, AliasStorageInterface $alias_storage_interface) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);

    $this->menuTree = $menu_tree;
    $this->activeTrail = $active_trail;
    $this->entityTypeManager = $entity_type_manager;
    $this->request = $current_request;
    $this->alias = $this->request->query->get('alias');
    if(is_numeric($this->request->query->get('max_depth'))){
      $this->maxDepth = (int)$this->request->query->get('max_depth');
    }
    $this->aliasManager = $alias_manager;
    $this->languageManager = $language_manager;
    $this->language = $this->languageManager->getCurrentLanguage()->getId();
    $this->aliasStorageInterface = $alias_storage_interface;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('menu.link_tree'),
      $container->get('menu.active_trail'),
      $container->get('entity_type.manager'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('path.alias_manager'),
      $container->get('language_manager'),
      $container->get('path.alias_storage')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function get(MenuInterface $menu) {
    // $cache_metadata = (new CacheableMetadata())->addCacheContexts(['url.query_args']);
    $available_languages = $this->languageManager->getLanguages();
    $alias_language = $this->language;
    $exists = FALSE;

    foreach ($available_languages as $langcode => $language) {
      if ($this->aliasStorageInterface->aliasExists($this->alias, $langcode)) {
        $exists = TRUE;
        $alias_language = $langcode;
      }
    }

    if ($exists) {
      $language = $this->languageManager->getLanguage($alias_language);
      $path = $this->aliasManager->getPathByAlias($this->alias, $alias_language);
      $url = Url::fromUri('internal:' . $path, ['language' => $language]);

      $active_trail_ids = $this->activeTrail->getActiveTrailIdsByRoute(
        $menu->id(), $url->getRouteName(), $url->getRouteParameters()
      );
    }
    else {
      $active_trail_ids = [];
    }

    $params = new MenuTreeParameters();
    $params->setActiveTrail($active_trail_ids);
    
    if (isset($this->maxDepth) && is_int($this->maxDepth)) {
      $params->setMaxDepth($this->maxDepth);
    }

    $tree = $this->menuTree->load($menu->id(), $params);

    $manipulators = [
      ['callable' => 'menu.default_tree_manipulators:checkNodeAccess'],
      ['callable' => 'menu.default_tree_manipulators:checkAccess'],
      ['callable' => 'menu.default_tree_manipulators:generateIndexAndSort'],
    ];

    $tree = $this->menuTree->transform($tree, $manipulators);

    // Maintain a clean version of the tree for adding cache metadata to the
    // response.
    $clean = $tree;

    // Remove the keys to prevent reordering.
    $this->removeKeys($tree);

    $response = new ResourceResponse($tree);
    
    // First add every item to the cache.
    $response->addCacheableDependency($menu);
    // $response->addCacheableDependency($cache_metadata);
    
    // Always use the clean version of the tree so every link is cached,
    // regardless of whether it's accessible to the current user or not.
    // $this->addCacheDependencies($clean, $response);

    return $response;
  }

  /**
   * Remove useless array keys.
   */
  protected function removeKeys(array &$data) {
    $tree = [];
    foreach ($data as $value) {
      if ($value->subtree) {
        $this->removeKeys($value->subtree);
      }

      $tree[] = $value;
    }

    $data = $tree;
  }

  /**
   * Add Cache Tags.
   */
  protected function addCacheDependencies(array $data,
                                          CacheableResponseInterface $response) {
    foreach ($data as $value) {
      // Gather the access cacheability of every item in the menu link tree,
      // including inaccessible items. This allows us to render cache the menu
      // tree, yet still automatically vary the rendered menu by the same cache
      // contexts that the access results vary by.
      // However, if $value->access is not an AccessResultInterface object, this
      // will still render the menu link, because this method does not want to
      // require access checking to be able to render a menu tree.
      if ($value->access instanceof AccessResultInterface) {
        $response->addCacheableDependency($value->access);
      }

      // Gather the cacheability of every item in the menu link tree. Some links
      // may be dynamic: they may have a dynamic text (e.g. a "Hi, <user>" link
      // text, which would vary by 'user' cache context), or a dynamic route
      // name or route parameters.
      $response->addCacheableDependency($value->link);

      // There may be additional dependencies that must be manually determined.
      $this->addLinkCacheDependencies($value->link, $response);

      if ($value->subtree) {
        $this->addCacheDependencies($value->subtree, $response);
      }
    }
  }

  /**
   * Add Cache Tags.
   */
  protected function addLinkCacheDependencies(MenuLinkInterface $link,
                                              CacheableResponseInterface $response) {
    $entity_type = NULL;
    $entity_type_id = $link->getBaseId();
    $uuid = $link->getDerivativeId();

    if ($link instanceof EntityInterface) {
      $entity_type = $link->getEntityType();
    }
    else {
      try {
        $entity_type = $this->entityTypeManager->getDefinition($entity_type_id);
      }
      catch (\Exception $e) {
        // Silence is golden.
      }
    }

    if (!$entity_type) {
      return;
    }

    // Add the list cache tags.
    $cache = new CacheableMetadata();
    $cache->addCacheTags($entity_type->getListCacheTags());
    $response->addCacheableDependency($cache);

    // If the link is an entity already, the cache tags were added in
    // ::addCacheDependencies().
    if ($link instanceof EntityInterface) {
      return;
    }

    // Get the entity.
    $entity = NULL;
    $storage = $this->entityTypeManager->getStorage($entity_type_id);
    $metadata = $link->getMetaData();

    if (!empty($metadata['entity_id'])) {
      $entity = $storage->load($metadata['entity_id']);
    }
    else {
      $entities = $storage->loadByProperties([
        $entity_type->getKey('uuid') => $uuid,
      ]);
      $entity = reset($entities);
    }

    if (!$entity) {
      return;
    }

    $response->addCacheableDependency($entity);
  }

  /**
   * {@inheritdoc}
   */
  protected function getBaseRoute($canonical_path, $method) {
    $route = parent::getBaseRoute($canonical_path, $method);

    $parameters = $route->getOption('parameters') ?: [];
    $parameters['menu']['type'] = 'entity:menu';
    $route->setOption('parameters', $parameters);

    return $route;
  }
}
