<?php

namespace Drupal\purest_user\Plugin\rest\resource;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Session\AccountInterface;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\user\UserInterface;
use Drupal\user\Plugin\rest\resource\UserRegistrationResource;
use Drupal\purest\Plugin\rest\resource\EntityResourceValidationTrait;
use Drupal\purest_user\AccountValidationServiceInterface;
use Drupal\purest_recaptcha\RecaptchaInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Provides a resource to get view modes by entity and bundle.
 *
 * @RestResource(
 *   id = "user_user_register_resource",
 *   label = @Translation("Purest User Register"),
 *   serialization_class = "Drupal\user\Entity\User",
 *   uri_paths = {
 *     "https://www.drupal.org/link-relations/create" = "/purest/user/register",
 *   }
 * )
 */
class RegisterResource extends UserRegistrationResource {

  use EntityResourceValidationTrait;

  /**
   * reCAPTCHA service
   */
  protected $recaptcha;

  /**
   * The current request
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $currentRequest;

  /**
   * The reCAPTCHA response
   */
  protected $recaptchaResponse;

  /**
   * The account validation service
   */
  protected $accountValidationService;

  /**
   * Constructs a new UserRegistrationResource instance.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param array $serializer_formats
   *   The available serialization formats.
   * @param \Psr\Log\LoggerInterface $logger
   *   A logger instance.
   * @param \Drupal\Core\Config\ImmutableConfig $user_settings
   *   A user settings config instance.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, ImmutableConfig $user_settings, AccountInterface $current_user, RecaptchaInterface $recaptcha, Request $current_request, AccountValidationServiceInterface $validation_service) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger, $user_settings, $current_user);
    $this->recaptcha = $recaptcha;
    $this->currentRequest = $current_request;
    $this->recaptchaResponse = $this->currentRequest->query->get('recaptcha');
    $this->accountValidationService = $validation_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('config.factory')->get('user.settings'),
      $container->get('current_user'),
      $container->get('purest_recaptcha.recaptcha'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('purest_user.validation')
    );
  }

  /**
   * Responds to POST requests.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity object.
   *
   * @return \Drupal\rest\ModifiedResourceResponse
   *   The HTTP response object.
   *
   * @throws \Symfony\Component\HttpKernel\Exception\HttpException
   *   Throws exception expected.
   */
  public function post(UserInterface $account = NULL) {

    if(!is_string($this->recaptchaResponse)){
      throw new BadRequestHttpException("reCAPTCHA query string must be present.");
    }

    $recaptcha_valid = $this->recaptcha->validate($this->recaptchaResponse);
    if(!$recaptcha_valid){
      throw new BadRequestHttpException("reCAPTCHA validation failed.");
    }
    
    $this->ensureAccountCanRegister($account);

    $account->set('init', $account->getEmail());
    $account->addRole('rest_user');
    $account->block();

    $this->checkEditFieldAccess($account);

    // Make sure that the user entity is valid (email and name are valid).
    $this->validate($account);

    // Create the account.
    $account->save();

    // Attempt to send account validation email
    $this->accountValidationService->sendVerificationEmail($account);

    return new ModifiedResourceResponse($account, 200);
  }

}
